/*
    This file is part of the KDE libraries
    SPDX-FileCopyrightText: 2019 Dan Leinir Turthra Jensen <admin@leinir.dk>

    SPDX-License-Identifier: LGPL-2.0-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#ifndef KNSHANDLERVERSION_H
#define KNSHANDLERVERSION_H

#include <QLatin1String>

static QLatin1String knshandlerversion("6.29.0");

#endif//KNSHANDLERVERSION_H
